@@ -213,7 +213,7 @@ pcl::SampleConsensusModelTorus<PointT, PointNT>::computeModelCoefficients(
     B << -d.dot(p0), -d.dot(p1), -d.dot(p2), -d.dot(p3);
 
     Eigen::Matrix<float, -1, -1> sol;
-    sol = A.bdcSvd(Eigen::ComputeThinU | Eigen::ComputeThinV).solve(B);
+    sol = A.jacobiSvd(Eigen::ComputeThinU | Eigen::ComputeThinV).solve(B);
 
     const float r_min = -sol(0);
     const float D = sol(1);
@@ -303,6 +303,7 @@ pcl::SampleConsensusModelTorus<PointT, PointNT>::selectWithinDistance(
     inliers.clear();
     return;
   }
+  const float squared_threshold = threshold * threshold;
   inliers.clear();
   error_sqr_dists_.clear();
   inliers.reserve(indices_->size());
@@ -315,13 +316,13 @@ pcl::SampleConsensusModelTorus<PointT, PointNT>::selectWithinDistance(
     Eigen::Vector3f torus_closest;
     projectPointToTorus(pt, pt_n, model_coefficients, torus_closest);
 
-    const float distance = (torus_closest - pt).norm();
+    const float distance = (torus_closest - pt).squaredNorm();
 
-    if (distance < threshold) {
+    if (distance < squared_threshold) {
       // Returns the indices of the points whose distances are smaller than the
       // threshold
       inliers.push_back((*indices_)[i]);
-      error_sqr_dists_.push_back(distance);
+      error_sqr_dists_.push_back(std::sqrt(distance));
     }
   }
 }
@@ -335,6 +336,7 @@ pcl::SampleConsensusModelTorus<PointT, PointNT>::countWithinDistance(
   if (!isModelValid(model_coefficients))
     return (0);
 
+  const float squared_threshold = threshold * threshold;
   std::size_t nr_p = 0;
 
   for (std::size_t i = 0; i < indices_->size(); ++i) {
@@ -344,9 +346,9 @@ pcl::SampleConsensusModelTorus<PointT, PointNT>::countWithinDistance(
     Eigen::Vector3f torus_closest;
     projectPointToTorus(pt, pt_n, model_coefficients, torus_closest);
 
-    const float distance = (torus_closest - pt).norm();
+    const float distance = (torus_closest - pt).squaredNorm();
 
-    if (distance < threshold) {
+    if (distance < squared_threshold) {
       nr_p++;
     }
   }
